@@ -4,8 +4,8 @@ from ipywidgets import (
 )
 
 from traitlets import (
-    Float, Unicode, Int, Tuple, List, Instance, Bool, Dict, link, observe,
-    default, validate, TraitError
+    Float, Unicode, Int, Tuple, List, Instance, Bool, Dict, Enum,
+    link, observe, default, validate, TraitError
 )
 
 from traittypes import Dataset
@@ -215,10 +215,10 @@ class Velocity(Layer):
     _view_name = Unicode('LeafletVelocityView').tag(sync=True)
     _model_name = Unicode('LeafletVelocityModel').tag(sync=True)
 
-    u_var = Unicode('')
-    v_var = Unicode('')
-    lat_dim = Unicode('latitude')
-    lon_dim = Unicode('longitude')
+    zonal_speed = Unicode('', help='Name of the zonal speed in the dataset')
+    meridional_speed = Unicode('', help='Name of the meridional speed in the dataset')
+    latitude_dimension = Unicode('latitude', help='Name of the latitude dimension in the dataset')
+    longitude_dimension = Unicode('longitude', help='Name of the longitude dimension in the dataset')
     units = Unicode(None, allow_none=True)
 
     data = Dataset().tag(sync=True, to_json=ds_x_to_json)
@@ -379,7 +379,7 @@ class LayerGroup(Layer):
         self.layers = ()
 
 
-class FeatureGroup(Layer):
+class FeatureGroup(LayerGroup):
     _view_name = Unicode('LeafletFeatureGroupView').tag(sync=True)
     _model_name = Unicode('LeafletFeatureGroupModel').tag(sync=True)
 
@@ -450,6 +450,79 @@ class LayersControl(Control):
     _model_name = Unicode('LeafletLayersControlModel').tag(sync=True)
 
 
+class MeasureControl(Control):
+    _view_name = Unicode('LeafletMeasureControlView').tag(sync=True)
+    _model_name = Unicode('LeafletMeasureControlModel').tag(sync=True)
+
+    _length_units = ['feet', 'meters', 'miles', 'kilometers']
+    _area_units = ['acres', 'hectares', 'sqfeet', 'sqmeters', 'sqmiles']
+    _custom_units_dict = {}
+    _custom_units = Dict().tag(sync=True)
+
+    position = Enum(
+        ['topright', 'topleft', 'bottomright', 'bottomleft'],
+        default_value='topright',
+        help="""Possible values are topleft, topright, bottomleft
+                or bottomright"""
+    ).tag(sync=True, o=True)
+
+    primary_length_unit = Enum(
+        values=_length_units,
+        default_value='feet',
+        help="""Possible values are feet, meters, miles, kilometers or any user
+                defined unit"""
+    ).tag(sync=True, o=True)
+
+    secondary_length_unit = Enum(
+        values=_length_units,
+        default_value=None,
+        allow_none=True,
+        help="""Possible values are feet, meters, miles, kilometers or any user
+                defined unit"""
+    ).tag(sync=True, o=True)
+
+    primary_area_unit = Enum(
+        values=_area_units,
+        default_value='acres',
+        help="""Possible values are acres, hectares, sqfeet, sqmeters, sqmiles
+                or any user defined unit"""
+    ).tag(sync=True, o=True)
+
+    secondary_area_unit = Enum(
+        values=_area_units,
+        default_value=None,
+        allow_none=True,
+        help="""Possible values are acres, hectares, sqfeet, sqmeters, sqmiles
+                or any user defined unit"""
+    ).tag(sync=True, o=True)
+
+    active_color = Color('#ABE67E').tag(sync=True, o=True)
+    completed_color = Color('#C8F2BE').tag(sync=True, o=True)
+
+    popup_options = Dict({
+      'className': 'leaflet-measure-resultpopup',
+      'autoPanPadding': [10, 10]
+    }).tag(sync=True, o=True)
+
+    capture_z_index = Int(10000).tag(sync=True, o=True)
+
+    def add_length_unit(self, name, factor, decimals=0):
+        self._length_units.append(name)
+        self._add_unit(name, factor, decimals)
+
+    def add_area_unit(self, name, factor, decimals=0):
+        self._area_units.append(name)
+        self._add_unit(name, factor, decimals)
+
+    def _add_unit(self, name, factor, decimals):
+        self._custom_units_dict[name] = {
+            'factor': factor,
+            'display': name,
+            'decimals': decimals
+        }
+        self._custom_units = dict(**self._custom_units_dict)
+
+
 class SplitMapControl(Control):
     _view_name = Unicode('LeafletSplitMapControlView').tag(sync=True)
     _model_name = Unicode('LeafletSplitMapControlModel').tag(sync=True)
@@ -479,12 +552,6 @@ class DrawControl(Control):
     _view_name = Unicode('LeafletDrawControlView').tag(sync=True)
     _model_name = Unicode('LeafletDrawControlModel').tag(sync=True)
 
-    layer = Instance(FeatureGroup).tag(sync=True, **widget_serialization)
-
-    @default('layer')
-    def _default_layer(self):
-        return FeatureGroup()
-
     # Enable each of the following drawing by giving them a non empty dict of options
     # You can add Leaflet style options in the shapeOptions sub-dict
     # See https://github.com/Leaflet/Leaflet.draw#polylineoptions
@@ -493,6 +560,7 @@ class DrawControl(Control):
     # See https://github.com/Leaflet/Leaflet.draw#polygonoptions
     # TODO: mutable default value!
     polygon = Dict({'shapeOptions': {}}).tag(sync=True)
+    circlemarker = Dict({'shapeOptions': {}}).tag(sync=True)
 
     # Leave empty to disable these
     circle = Dict().tag(sync=True)
@@ -525,6 +593,27 @@ class DrawControl(Control):
     def on_draw(self, callback, remove=False):
         self._draw_callbacks.register_callback(callback, remove=remove)
 
+    def clear(self):
+        self.send({'msg': 'clear'})
+
+    def clear_polylines(self):
+        self.send({'msg': 'clear_polylines'})
+
+    def clear_polygons(self):
+        self.send({'msg': 'clear_polygons'})
+
+    def clear_circles(self):
+        self.send({'msg': 'clear_circles'})
+
+    def clear_circle_markers(self):
+        self.send({'msg': 'clear_circle_markers'})
+
+    def clear_rectangles(self):
+        self.send({'msg': 'clear_rectangles'})
+
+    def clear_markers(self):
+        self.send({'msg': 'clear_markers'})
+
 
 class Map(DOMWidget, InteractMixin):
     _view_name = Unicode('LeafletMapView').tag(sync=True)
